use crate::vm::object_type::DmImplementation;
use std::alloc::{alloc, dealloc, Layout};
use std::collections::HashSet;
use std::rc::Rc;

#[derive(Debug, PartialEq, Eq)]
pub struct DvmObject {
    pub data: *mut u8,
    pub units: HashSet<usize>,
    pub size: usize,
    pub layout: Layout,
    pub implementation: Rc<DmImplementation>,
}

impl DvmObject {
    pub fn new(size: usize, implementation: Rc<DmImplementation>) -> Self {
        let layout = Layout::from_size_align(size, 1).unwrap();
        DvmObject {
            data: unsafe { alloc(layout) },
            units: HashSet::new(),
            size,
            layout,
            implementation,
        }
    }
}

impl Drop for DvmObject {
    fn drop(&mut self) {
        unsafe {
            dealloc(self.data, self.layout);
        }
    }
}
